package cn.com.qmth.examcloud.core.reports.api.controller;

import java.util.List;

import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.reports.base.bean.ExamOrgMainTopTen;
import cn.com.qmth.examcloud.core.reports.base.bean.ExamOrgReportMainBean;
import cn.com.qmth.examcloud.core.reports.base.bean.ExamReportMainBean;
import cn.com.qmth.examcloud.core.reports.base.enums.ReportStatus;
import cn.com.qmth.examcloud.core.reports.dao.entity.ProjectEntity;
import cn.com.qmth.examcloud.core.reports.service.ExamOrgReportService;
import cn.com.qmth.examcloud.core.reports.service.ProjectService;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@RestController
@Api(tags = "考试-学习中心报表")
@RequestMapping("${$rmp.ctr.reports}/" + "examOrgReport")
public class ExamOrgReportController extends ControllerSupport {
	@Autowired
	private ExamOrgReportService examOrgReportService;
	@Autowired
	private ProjectService projectService;


	@ApiOperation(value = "查询考试数据")
	@GetMapping(value = "/getExamMainList")
	public List<ExamReportMainBean> getExamMainList(@RequestParam Long projectId,
			@RequestParam(required = false) Long examId) {
		if (projectId == null) {
			throw new StatusException("100001", "projectId不能为空");
		}
		User user = getAccessUser();
		ProjectEntity pe = projectService.findProjectById(projectId);
		if (pe == null) {
			throw new StatusException("100002", "未找到项目");
		}
		if (!user.getRootOrgId().equals(pe.getRootOrgId())) {
			throw new StatusException("100003", "非法操作");
		}
		if (!ReportStatus.SUCCESS.equals(pe.getReportStatus())) {
			throw new StatusException("100004", "只能查看报表计算成功的项目");
		}
		return examOrgReportService.getExamMainList(projectId, examId);
	}

	@ApiOperation(value = "查询考试-中心数据")
	@GetMapping(value = "/getExamOrgMainList")
	public List<ExamOrgReportMainBean> getExamOrgMainList(@RequestParam Long projectId,
			@RequestParam(required = false) Long examId) {
		if (projectId == null) {
			throw new StatusException("100011", "projectId不能为空");
		}
		User user = getAccessUser();
		ProjectEntity pe = projectService.findProjectById(projectId);
		if (pe == null) {
			throw new StatusException("100012", "未找到项目");
		}
		if (!user.getRootOrgId().equals(pe.getRootOrgId())) {
			throw new StatusException("100013", "非法操作");
		}
		if (!ReportStatus.SUCCESS.equals(pe.getReportStatus())) {
			throw new StatusException("100014", "只能查看报表计算成功的项目");
		}
		return examOrgReportService.getExamOrgMainList(projectId, examId);
	}

	@ApiOperation(value = "查询考试-中心数据Top10")
	@GetMapping(value = "/getExamOrgMainTop10")
	public ExamOrgMainTopTen getExamOrgMainTop10(@RequestParam Long projectId,
			@RequestParam(required = false) Long examId) {
		if (projectId == null) {
			throw new StatusException("100021", "projectId不能为空");
		}
		User user = getAccessUser();
		ProjectEntity pe = projectService.findProjectById(projectId);
		if (pe == null) {
			throw new StatusException("100022", "未找到项目");
		}
		if (!user.getRootOrgId().equals(pe.getRootOrgId())) {
			throw new StatusException("100023", "非法操作");
		}
		if (!ReportStatus.SUCCESS.equals(pe.getReportStatus())) {
			throw new StatusException("100024", "只能查看报表计算成功的项目");
		}
		return examOrgReportService.getExamOrgMainTop10(projectId, examId);
	}
	@ApiOperation(value = "导出数据")
	@GetMapping(value = "/export")
	public void exportAll(@RequestParam Long projectId, @RequestParam(required = false) Long examId,
			@RequestParam(required = false) String items, HttpServletResponse response) {
		if (projectId == null) {
			throw new StatusException("100031", "projectId不能为空");
		}
		User user = getAccessUser();
		ProjectEntity pe = projectService.findProjectById(projectId);
		if (pe == null) {
			throw new StatusException("100032", "未找到项目");
		}
		if (!user.getRootOrgId().equals(pe.getRootOrgId())) {
			throw new StatusException("100033", "非法操作");
		}
		if (!ReportStatus.SUCCESS.equals(pe.getReportStatus())) {
			throw new StatusException("100034", "只能导出报表计算成功的项目");
		}
		examOrgReportService.exportAll(pe,examId,items, response);
	}
}