package cn.com.qmth.examcloud.core.reports.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.criteria.Predicate;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.api.commons.exchange.PageInfo;
import cn.com.qmth.examcloud.api.commons.security.bean.UserType;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.DateUtil;
import cn.com.qmth.examcloud.core.reports.base.bean.QueryStudentOperateReq;
import cn.com.qmth.examcloud.core.reports.base.bean.StudentOperateBean;
import cn.com.qmth.examcloud.core.reports.dao.OperateRepo;
import cn.com.qmth.examcloud.core.reports.dao.entity.OperateEntity;
import cn.com.qmth.examcloud.core.reports.service.OperateService;
import cn.com.qmth.examcloud.reports.commons.bean.OperateReport;

@Service
public class OperateServiceImpl implements OperateService {
	@Autowired
	private OperateRepo operateRepo;
	
	
	@Transactional
	@Override
	public void saveOperate(OperateReport r, String msgId) {
		OperateEntity e=new OperateEntity();
		e.setMsgId(msgId);
		e.setRootOrgId(r.getRootOrgId());
		e.setStudentId(r.getStudentId());
		e.setOperateUserId(r.getOperateUserId());
		e.setOperateUserType(r.getOperateUserType());;
		e.setOperateTime(r.getReportTime());
		e.setOperateIp(r.getRemoteHost());
		e.setOperate(r.getOperate());
		e.setExamStudentId(r.getExamStudentId());
		e.setCreationTime(new Date());
		try {
			operateRepo.save(e);
		} catch (DuplicateKeyException e1) {
			//忽略;
		}
	}


	@Override
	public PageInfo<StudentOperateBean> queryPage(QueryStudentOperateReq req, Integer pageNo, Integer pageSize,
			Long rootOrgId) {
		Long studentId = req.getStudentId();
		if(studentId==null) {
			throw new StatusException("10001", "studentId不能为空");
		}
		String operate = req.getOperate();
		String startTime=req.getStartTime();
		String endTime=req.getEndTime();
		Specification<OperateEntity> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();
			predicates.add(cb.equal(root.get("rootOrgId"), rootOrgId));
			predicates.add(cb.equal(root.get("studentId"), studentId));

			if(StringUtils.isNotBlank(operate)) {
				predicates.add(cb.like(root.get("operate"), toSqlSearchPattern(operate)));
			}
			if (StringUtils.isNotBlank(startTime)) {
				Date start=DateUtil.parse(startTime, "yyyy/MM/dd HH:mm:ss");
				predicates.add(cb.greaterThanOrEqualTo(root.get("operateTime"), start));
			}
			if (StringUtils.isNotBlank(endTime)) {
				Date end=DateUtil.parse(endTime, "yyyy/MM/dd HH:mm:ss");
				predicates.add(cb.lessThanOrEqualTo(root.get("operateTime"), end));
			}

			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};

		PageRequest pageRequest = PageRequest.of(pageNo - 1, pageSize, new Sort(Direction.DESC, "operateTime"));

		Page<OperateEntity> data = operateRepo.findAll(specification, pageRequest);

		List<StudentOperateBean> ret = Lists.newArrayList();
		String studentClient="考生端";
		String adminClient="管理端";
		for (OperateEntity cur : data) {
			StudentOperateBean bean = new StudentOperateBean();
			bean.setOperate(cur.getOperate());
			bean.setOperateIp(cur.getOperateIp());
			bean.setOperateTime(cur.getOperateTime());
			if(UserType.COMMON.equals(cur.getOperateUserType())) {
				bean.setOperateClient(adminClient);
			}
			if(UserType.STUDENT.equals(cur.getOperateUserType())) {
				bean.setOperateClient(studentClient);
			}
			ret.add(bean);
		}
		return new PageInfo<StudentOperateBean>(data, ret);
	}
	private String toSqlSearchPattern(String column) {
		if (StringUtils.isBlank(column)) {
			return "%";
		} else {
			column = column.trim().replaceAll("\\s", "%");
			column = "%" + column + "%";
			return column;
		}
	}


	@Transactional
	@Override
	public void clean() {
		operateRepo.clean();
	}
}
